program comparedata
implicit real*8 (a-h,o-z)
logical alive
character c80tmp*80,c80tmp2*80,file1*200,file2*200
real*8 MAPE
real*8,allocatable :: data1(:),data2(:)

write(*,*) "comparedata: Calculate various statistical quantities between two set of data"
write(*,*) "Programmed by Tian Lu (sobereva@sina.com)"
write(*,*) "Release date: 2023-Jun-20"
write(*,*)

call getarg(1,file1)
call getarg(2,file2)

!file1="DFT-SAPT.txt"
!file2="SAPT2+.txt"
if (trim(file1)=="".or.trim(file2)=="") then
    write(*,"(a)") " Input path of the file containing the first set of data (reference data)"
    write(*,*) "For example, D:\sob\Yohane.txt"
    do while(.true.)
	    read(*,"(a)") file1
	    inquire(file=file1,exist=alive)
	    if (alive) exit
	    write(*,*) "Cannot find the file, input again!"
    end do
    write(*,*) "Input path of the file containing the second set of data"
    write(*,*) "For example, D:\sob\Nozomi.txt"
    do while(.true.)
	    read(*,"(a)") file2
	    inquire(file=file2,exist=alive)
	    if (alive) exit
	    write(*,*) "Cannot find the file, input again!"
    end do
end if

open(10,file=file1,status="old")
ndata=totlinenum(10,1)
write(*,"(' Number of data to load in each file:',i12)") ndata
write(*,*) "Loading file 1..."
allocate(data1(ndata),data2(ndata))
rewind(10)
do i=1,ndata
    read(10,*) data1(i)
end do
close(10)
write(*,*) "Loading file 2..."
open(10,file=file2,status="old")
do i=1,ndata
    read(10,*) data2(i)
end do
close(10)
write(*,*) "Loading finished!"

abserrmin=1D200
abserrmax=0
absrelerrmin=1D200
absrelerrmax=0
errmean=0
abserrmean=0
errsqr=0
MAPE=0
data1sumsqr=0
do i=1,ndata
	err=data2(i)-data1(i)
	abserr=abs(err)
    relerr=err/data1(i)
    absrelerr=abs(relerr)
	if (abserr<abserrmin) then
		abserrmin=abserr
        iabserrmin=i
        errmin=err
    end if
    if (abserr>abserrmax) then
		abserrmax=abserr
        iabserrmax=i
        errmax=err
    end if
	if (absrelerr<absrelerrmin) then
		absrelerrmin=absrelerr
        iabsrelerrmin=i
        relerrmin=relerr
    end if
	if (absrelerr>absrelerrmax) then
		absrelerrmax=absrelerr
        iabsrelerrmax=i
        relerrmax=relerr
    end if
    errmean=errmean+err
    abserrmean=abserrmean+abserr
    errsqr=errsqr+err**2
    data1sumsqr=data1sumsqr+data1(i)**2
    MAPE=MAPE+absrelerr
end do
errmean=errmean/ndata
abserrmean=abserrmean/ndata
RMSE=dsqrt(errsqr/ndata)
RRMSE=dsqrt(errsqr/data1sumsqr)
MAPE=MAPE/ndata
pearsoncoeff=covarray(data1(:),data2(:)) / stddevarray(data1(:))/stddevarray(data2(:))

write(*,*)
write(*,*) "Note: The errors reported below are for data 2 relative to data 1"
write(*,*)
write(*,"(' Minimum error:',f30.15,' at index',i10)") errmin,iabserrmin
write(*,"(' Maximum error:',f30.15,' at index',i10)") errmax,iabserrmax
write(*,"(' Minimum relative error:',f12.5,' % at index',i10)") relerrmin*100,iabsrelerrmin
write(*,"(' Maximum relative error:',f12.5,' % at index',i10)") relerrmax*100,iabsrelerrmax
write(*,"(' Minimum absolute error:',f30.15,' at index',i10)") abserrmin,iabserrmin
write(*,"(' Maximum absolute error:',f30.15,' at index',i10)") abserrmax,iabserrmax
write(*,"(' Minimum absolute relative error:',f12.5,' % at index',i10)") absrelerrmin*100,iabsrelerrmin
write(*,"(' Maximum absolute relative error:',f12.5,' % at index',i10)") absrelerrmax*100,iabsrelerrmax
write(*,"(' Mean error (ME):',f30.15)") errmean
write(*,"(' Mean absolute error (MAE):',f30.15)") abserrmean
write(*,"(' Root mean square error (RMSE):',f30.15)") RMSE
write(*,"(' Relative root mean square error (RRMSE):',f30.15,' %')") RRMSE*100
write(*,"(' Mean absolute percentage error (MAPE):',f12.5,' %')") MAPE*100
write(*,"(2(a,f12.6))") " Pearson correlation coefficient r:",pearsoncoeff,"  r^2:",pearsoncoeff**2

!write(*,*)
!write(*,*) "Press ENTER button to exit"
!read(*,*)


contains


!!--------------------- Determine how many lines in the fileid
!If imode==1, space line will be regarded as the sign of end of file. If imode==2, will count actual number of lines in the file
integer function totlinenum(fileid,imode)
integer fileid,ierror,imode
character c80*80
totlinenum=0
rewind(fileid)
do while(.true.)
	read(fileid,"(a)",iostat=ierror) c80
	if (imode==1) then
		if (ierror/=0.or.c80==" ") exit
	else if (imode==2) then
		if (ierror/=0) exit
	end if
	totlinenum=totlinenum+1
end do
rewind(fileid)
end function


real*8 function stddevarray(array)
real*8 array(:),avg
avg=sum(array)/size(array)
stddevarray=dsqrt(sum((array-avg)**2)/size(array))
end function


real*8 function covarray(array1,array2)
real*8 array1(:),array2(:),avg1,avg2
avg1=sum(array1)/size(array1)
avg2=sum(array2)/size(array2)
covarray=sum((array1-avg1)*(array2-avg2))/size(array1)
end function

    
end program